package net.javase.onenet.api;

import cn.hutool.http.ContentType;
import cn.hutool.http.Method;
import net.javase.onenet.config.Config;
import net.javase.onenet.enums.ApiEnum;
import net.javase.onenet.enums.SignMethod;
import net.javase.onenet.model.FuseLbsModel;
import net.javase.onenet.model.FuseOtaModel;
import net.javase.onenet.model.RegistDeviceIdentityModel;
import net.javase.onenet.model.VoiceNotifyModel;
import net.javase.onenet.utils.RequestUtil;

import java.io.File;
import java.util.Map;

/**
 * FuseApi
 *
 * @author Frank
 */
public class FuseApi {

    /**
     * <a href="https://open.iot.10086.cn/doc/v5/fuse/detail/1432">批量自动注册标识</a>
     *
     * @param config {@link Config}
     * @param body   http body参数
     * @return {@link ApiResponse}
     */
    public static ApiResponse batchAutoRegistDeviceIdentity(Config config, Map<String, Object> body) {
        return ApiRequest.builder()
                .api(ApiEnum.FUSE_IDENTITY_DEVICE.api())
                .method(Method.POST)
                .contentType(ContentType.JSON)
                .signMethod(SignMethod.MD5)
                .body(body)
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * <a href="https://open.iot.10086.cn/doc/v5/fuse/detail/1432">批量自动注册标识</a>
     *
     * @param config {@link Config}
     * @param model  {@link RegistDeviceIdentityModel}
     * @return {@link ApiResponse}
     */
    public static ApiResponse batchAutoRegistDeviceIdentity(Config config, RegistDeviceIdentityModel model) {
        return batchAutoRegistDeviceIdentity(config, model.toMap());
    }

    /**
     * <a href="https://open.iot.10086.cn/doc/v5/fuse/detail/1432">批量自动注册标识</a>
     *
     * @param config {@link Config}
     * @param body   http body参数
     * @return {@link ApiResponse}
     */
    public static ApiResponse voiceNotify(Config config, Map<String, Object> body) {
        return ApiRequest.builder()
                .api(ApiEnum.VOICE_NOTIFY.api())
                .method(Method.POST)
                .contentType(ContentType.JSON)
                .signMethod(SignMethod.MD5)
                .body(body)
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * <a href="https://open.iot.10086.cn/doc/v5/fuse/detail/1432">批量自动注册标识</a>
     *
     * @param config {@link Config}
     * @param model  {@link VoiceNotifyModel}
     * @return {@link ApiResponse}
     */
    public static ApiResponse voiceNotify(Config config, VoiceNotifyModel model) {
        return voiceNotify(config, model.toMap());
    }

    /**
     * <a href="https://open.iot.10086.cn/doc/v5/fuse/detail/1437">基站定位获取最新位置</a>
     *
     * @param config {@link Config}
     * @param query  http query参数
     * @return {@link ApiResponse}
     */
    public static ApiResponse latestLocation(Config config, Map<String, Object> query) {
        return ApiRequest.builder()
                .api(ApiEnum.LBS_LATEST_LOCATION.api())
                .method(Method.GET)
                .signMethod(SignMethod.MD5)
                .query(query)
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * <a href="https://open.iot.10086.cn/doc/v5/fuse/detail/1437">基站定位获取最新位置</a>
     *
     * @param config {@link Config}
     * @param model  {@link FuseLbsModel}
     * @return {@link ApiResponse}
     */
    public static ApiResponse latestLocation(Config config, FuseLbsModel model) {
        return latestLocation(config, model.toMap());
    }

    /**
     * <a href="https://open.iot.10086.cn/doc/v5/fuse/detail/1438">基站定位历史轨迹查询</a>
     *
     * @param config {@link Config}
     * @param query  http query参数
     * @return {@link ApiResponse}
     */
    public static ApiResponse getTrail(Config config, Map<String, Object> query) {
        return ApiRequest.builder()
                .api(ApiEnum.LBS_GET_TRAIL.api())
                .method(Method.GET)
                .signMethod(SignMethod.MD5)
                .query(query)
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * <a href="https://open.iot.10086.cn/doc/v5/fuse/detail/1438">基站定位历史轨迹查询</a>
     *
     * @param config {@link Config}
     * @param model  {@link FuseLbsModel}
     * @return {@link ApiResponse}
     */
    public static ApiResponse getTrail(Config config, FuseLbsModel model) {
        return getTrail(config, model.toMap());
    }

    /**
     * <a href="https://open.iot.10086.cn/doc/v5/fuse/detail/1439">WIFI定位获取最新位置</a>
     *
     * @param config {@link Config}
     * @param query  http query参数
     * @return {@link ApiResponse}
     */
    public static ApiResponse latestWifiLocation(Config config, Map<String, Object> query) {
        return ApiRequest.builder()
                .api(ApiEnum.LBS_LATEST_WIFI_LOCATION.api())
                .method(Method.GET)
                .signMethod(SignMethod.MD5)
                .query(query)
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * <a href="https://open.iot.10086.cn/doc/v5/fuse/detail/1439">WIFI定位获取最新位置</a>
     *
     * @param config {@link Config}
     * @param model  {@link FuseLbsModel}
     * @return {@link ApiResponse}
     */
    public static ApiResponse latestWifiLocation(Config config, FuseLbsModel model) {
        return latestWifiLocation(config, model.toMap());
    }

    /**
     * <a href="https://open.iot.10086.cn/doc/v5/fuse/detail/1443">WIFI定位历史轨迹查询</a>
     *
     * @param config {@link Config}
     * @param query  http query参数
     * @return {@link ApiResponse}
     */
    public static ApiResponse getWifiTrail(Config config, Map<String, Object> query) {
        return ApiRequest.builder()
                .api(ApiEnum.LBS_GET_WIFI_TRAIL.api())
                .method(Method.GET)
                .signMethod(SignMethod.MD5)
                .query(query)
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * <a href="https://open.iot.10086.cn/doc/v5/fuse/detail/1443">WIFI定位历史轨迹查询</a>
     *
     * @param config {@link Config}
     * @param model  {@link FuseLbsModel}
     * @return {@link ApiResponse}
     */
    public static ApiResponse getWifiTrail(Config config, FuseLbsModel model) {
        return getWifiTrail(config, model.toMap());
    }

    /**
     * <a href="https://open.iot.10086.cn/doc/v5/fuse/detail/1447">检测升级任务</a>
     *
     * @param config {@link Config}
     * @param model  {@link FuseOtaModel}
     * @return {@link ApiResponse}
     */
    public static ApiResponse otaCheck(Config config, FuseOtaModel model) {
        return ApiRequest.builder()
                .api(ApiEnum.OTA_CHECK.api(model.getProId(), model.getDevName()))
                .method(Method.GET)
                .signMethod(SignMethod.MD5)
                .query(model.toMap())
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * <a href="https://open.iot.10086.cn/doc/v5/fuse/detail/1448">下载升级包</a>
     *
     * @param config {@link Config}
     * @param model  {@link FuseOtaModel}
     * @param target {@link File}
     */
    public static void otaDownload(Config config, FuseOtaModel model, File target) {
        RequestUtil.download(ApiRequest.builder()
                .api(ApiEnum.OTA_DOWNLOAD.api(model.getProId(), model.getDevName(), model.getTid()))
                .method(Method.GET)
                .signMethod(SignMethod.MD5)
                .query(model.toMap())
                .userid(config.getUserid())
                .build(), config.getAccessKey(), target);
    }

    /**
     * <a href="https://open.iot.10086.cn/doc/v5/fuse/detail/1449">上报升级状态</a>
     *
     * @param config {@link Config}
     * @param model  {@link FuseOtaModel}
     * @return {@link ApiResponse}
     */
    public static ApiResponse otaStatus(Config config, FuseOtaModel model) {
        return ApiRequest.builder()
                .api(ApiEnum.OTA_STATUS.api(model.getProId(), model.getDevName(), model.getTid()))
                .method(Method.POST)
                .signMethod(SignMethod.MD5)
                .query(model.toMap())
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }


    /**
     * <a href="https://open.iot.10086.cn/doc/v5/fuse/detail/1450">检测升级状态</a>
     *
     * @param config {@link Config}
     * @param model  {@link FuseOtaModel}
     * @return {@link ApiResponse}
     */
    public static ApiResponse otaTidCheck(Config config, FuseOtaModel model) {
        return ApiRequest.builder()
                .api(ApiEnum.OTA_TID_CHECK.api(model.getProId(), model.getDevName(), model.getTid()))
                .method(Method.GET)
                .signMethod(SignMethod.MD5)
                .query(model.toMap())
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * <a href="https://open.iot.10086.cn/doc/v5/fuse/detail/1451">查看设备版本号</a>
     *
     * @param config {@link Config}
     * @param model  {@link FuseOtaModel}
     * @return {@link ApiResponse}
     */
    public static ApiResponse otaVersion(Config config, FuseOtaModel model) {
        return ApiRequest.builder()
                .api(ApiEnum.OTA_VERSION.api(model.getProId(), model.getDevName()))
                .method(Method.GET)
                .signMethod(SignMethod.MD5)
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * <a href="https://open.iot.10086.cn/doc/v5/fuse/detail/1454">上报设备版本号</a>
     *
     * @param config {@link Config}
     * @param model  {@link FuseOtaModel}
     * @return {@link ApiResponse}
     */
    public static ApiResponse otaVersionUp(Config config, FuseOtaModel model) {
        return ApiRequest.builder()
                .api(ApiEnum.OTA_VERSION.api(model.getProId(), model.getDevName()))
                .method(Method.POST)
                .signMethod(SignMethod.MD5)
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

}
