package net.javase.onenet.model;

import lombok.Builder;
import lombok.Getter;
import lombok.Setter;
import net.javase.onenet.annotation.ApiParam;
import net.javase.onenet.model.base.BaseModel;

import java.util.List;

/**
 * VoiceNotifyModel
 *
 * @author Frank
 */
@Getter
@Setter
@Builder
public class VoiceNotifyModel extends BaseModel {

    /**
     * 外呼被叫号码MSISDN <br>
     * 举例8618625150488
     */
    @ApiParam(name = "participant_address")
    private String participantAddress;

    /**
     * 号显号码（业务号码）
     * 由OneNET平台提供
     */
    private String display;

    /**
     * 呼叫操作指示，参考CallActionType定义
     * 仅支持Play和PlayAndCollect
     */
    private List<CallActionType> actions;

    /**
     * 用户指定的通知地址（请求方式为POST）
     * 如无此字段，则根据OneNET平台配置为准
     */
    @ApiParam(name = "notify_url")
    private String notifyUrl;


    /**
     * CallActionType 呼叫操作类型
     *
     * @author Frank
     */
    @Getter
    @Setter
    @Builder
    public static class CallActionType {
        /**
         * 操作类型
         * Play：放音
         * PlayAndCollect：放音及收号
         */
        private String operation;

        /**
         * tts模板编号
         * 用户向OneNET提交的通知模板，需审核通过后才能使用
         */
        private String tts_template;

        /**
         * 模板中需要填充的参数参数内容为UTF-8编码，JSON Object的格式为{"var1":"aa","var2":"aa"}，其中var1，var2为模板变量名称，var的个数可变
         */
        private String param_value;

        /**
         * 收号长度当operation=PlayAndCollect时有效，如果replayAfterCollection=false或未配置，系统会将用户输入的号码通过呼叫状态通知API发送给第三方；如果replayAfterCollection=true，则不会将收号结果上报给第三方
         * 取值范围
         * 1~32：收号，取值表示收号位长，最大支持32位
         */
        private String collect_length;

        /**
         * 重新播放标识
         * 当operation=PlayAndCollect时有效，此字段用于设置是否在收号后重新播放指定的放音
         * true：播放
         * false：不播放
         * 默认值：false
         */
        private String replay_after_collection;

        /**
         * 当replayAfterCollection=true时此参数有效
         * 此字段用于设置触发重新放音的收号内容，只有实际收号内容与该参数值一致时，才重新放音
         * 例如：该参数设置为1，当用户根据收号语音提示按1，两者一致，则重复放音默认值为1
         */
        private String collect_content_trigger_replaying;

        /**
         * 重复放音次数
         * 当操作类型为Play时，此字段有效
         * 默认为1次
         */
        private String relay_time;

    }
}

